@@ -458,6 +458,13 @@ class DatabaseRelationAdapter(OpenStackRelationAdapter):
         """
         return self.relation.db_host()
 
+    @property
+    def port(self):
+        """
+        Port that should be used to access a database.
+        """
+        return self.relation.db_port()
+
     @property
     def type(self):
         return 'mysql'
@@ -475,20 +482,30 @@ class DatabaseRelationAdapter(OpenStackRelationAdapter):
                 ch_utils.OPENSTACK_RELEASES.index('stein')):
             driver = 'mysql+pymysql'
         if prefix:
-            uri = '{}://{}:{}@{}/{}'.format(
+            username = self.relation.username(prefix=prefix)
+            password = self.relation.password(prefix=prefix)
+            database = self.relation.database(prefix=prefix)
+        else:
+            username = self.username
+            password = self.password
+            database = self.database
+        if self.port:
+            uri = '{}://{}:{}@{}:{}/{}'.format(
                 driver,
-                self.relation.username(prefix=prefix),
-                self.relation.password(prefix=prefix),
+                username,
+                password,
                 self.host,
-                self.relation.database(prefix=prefix),
+                self.port,
+                database,
             )
         else:
+            # defensive code if port is not passed
             uri = '{}://{}:{}@{}/{}'.format(
                 driver,
-                self.username,
-                self.password,
+                username,
+                password,
                 self.host,
-                self.database,
+                database,
             )
         try:
             if self.ssl_ca:
