@@ -472,11 +472,14 @@ class BaseOpenStackCharm(object, metaclass=BaseOpenStackCharmMeta):
         return version_or_codename
 
     @staticmethod
-    def get_os_codename_package(package, codenames, fatal=True):
-        """Derive OpenStack release codename from an installed package.
+    def get_os_codename_package(package, codenames, fatal=True,
+                                apt_cache_sufficient=False):
+        """Derive OpenStack release codename from a package.
 
-        :param package: str Package name to lookup (ie. in apt cache)
-        :param codenames: dict of OrderedDict eg (not applicable for snap pkgs)
+        :param package: Package name to lookup (ie. in apt cache)
+        :type package: str
+        :param codenames: Map of package to (version, os_release) tuples.
+            Example:
             {
              'pkg1': collections.OrderedDict([
                  ('2', 'mitaka'),
@@ -487,8 +490,19 @@ class BaseOpenStackCharm(object, metaclass=BaseOpenStackCharmMeta):
                  ('13.2', 'newton'),
                  ('14.7', 'ocata'), ]),
             }
-        :param fatal: bool Raise exception if pkg not installed
-        :returns: str OpenStack version name corresponding to package
+        :type codenames: Dict[str,collections.OrderedDict[Tuple(str,str)]]
+        :param fatal: Raise exception if pkg not installed
+        :type fatal: bool
+        :param apt_cache_sufficient: When False (the default) version from an
+            installed package will be used, when True version from the systems
+            APT cache will be used.  This is useful for subordinate charms who
+            need working release selection prior to package installation and
+            has no way of using fall back to version of a package the principle
+            charm has installed nor package source configuration option.
+        :type apt_cache_sufficient: bool
+        :returns: OpenStack version name corresponding to package
+        :rtype: Optional[str]
+        :raises: AttributeError, ValueError
         """
         cache = fetch.apt_cache()
 
@@ -498,14 +512,19 @@ class BaseOpenStackCharm(object, metaclass=BaseOpenStackCharmMeta):
             if not fatal:
                 return None
             # the package is unknown to the current apt cache.
-            e = ('Could not determine version of package with no installation '
-                 'candidate: {}'.format(package))
-            raise Exception(e)
-        if not pkg.current_ver:
-            if not fatal:
-                return None
+            e = ValueError(
+                'Could not determine version of package with no installation '
+                'candidate: {}'.format(package))
+            raise e
+
+        if apt_cache_sufficient:
+            vers = fetch.apt_pkg.upstream_version(pkg.version)
+        else:
+            if not pkg.current_ver:
+                if not fatal:
+                    return None
+            vers = fetch.apt_pkg.upstream_version(pkg.current_ver.ver_str)
 
-        vers = fetch.apt_pkg.upstream_version(pkg.current_ver.ver_str)
         # x.y match only for 20XX.X
         # and ignore patch level for other packages
         match = re.match('^(\d+)\.(\d+)', vers)
@@ -631,19 +650,26 @@ class BaseOpenStackCharmActions(object):
         # AJK: we set this as charms can use it to detect installed state
         self.set_state('{}-installed'.format(self.name))
         self.update_api_ports()
-        hookenv.status_set('maintenance',
-                           'Installation complete - awaiting next status')
+        if packages:
+            # NOTE(fnordahl): Update status only if we actually performed
+            # package installation to avoid uneccessary status transitions.
+            # LP: #1861775
+            hookenv.status_set('maintenance',
+                               'Installation complete - awaiting next status')
 
-    def configure_source(self):
-        """Configure installation source using the config item
-        indicated in the source_config_key class variable
-        (default: 'openstack-origin')
+    def configure_source(self, config_key=None):
+        """Configure installation source.
 
-        This configures the installation source for deb packages and then
-        updates the packages list on the unit.
+        :param config_key: Config item (default: value indicated in the
+                           source_config_key class variable)
+        :type config_key: Optional[str]
+
+        This adds an installation source for deb packages and then updates the
+        packages list on the unit.
         """
+        config_key = config_key or self.source_config_key
         source, key = os_utils.get_source_and_pgp_key(
-            self.config[self.source_config_key])
+            self.config[config_key])
         fetch.add_source(source, key)
         fetch.apt_update(fatal=True)
 
@@ -771,16 +797,25 @@ class BaseOpenStackCharmActions(object):
                     # means we need to skip this config to avoid rendering
                     return
 
-                charmhelpers.core.templating.render(
-                    source=os.path.basename(conf),
-                    template_loader=os_templating.get_loader(
-                        'templates/', self.release),
-                    target=conf,
-                    context=adapters_instance,
-                    config_template=config_template,
-                    group=self.group,
-                    perms=self.permission_override_map.get(conf) or 0o640,
-                )
+                def _render(source):
+                    charmhelpers.core.templating.render(
+                        source=source,
+                        template_loader=os_templating.get_loader(
+                            'templates/', self.release),
+                        target=conf,
+                        context=adapters_instance,
+                        config_template=config_template,
+                        group=self.group,
+                        perms=self.permission_override_map.get(conf) or 0o640,
+                    )
+                try:
+                    _render(os.path.basename(conf))
+                except LookupError:
+                    # if no template is found try looking for files named after
+                    # the absolute path of target with path separators replaced
+                    # by underscores.  This convention is useful when charm
+                    # author need to provide templates with ambiguous basenames
+                    _render('_'.join(conf.split(os.path.sep))[1:])
 
     def render_with_interfaces(self, interfaces, configs=None):
         """Render the configs using the interfaces passed; overrides any
@@ -906,6 +941,16 @@ class BaseOpenStackCharmActions(object):
             cur_vers = self.get_os_version_snap(snap)
         else:
             cur_vers = self.get_os_version_package(package)
+
+        if cur_vers is None or avail_vers is None:
+            raise RuntimeError(
+                "In charms_openstack.charm.core.openstack_upgrade_available() "
+                "cur_vers={} and avail_vers={}, one of which is None. "
+                "This usually implies that the openstack version is not "
+                "present in the self.package_codenames or "
+                "os_utils.PACKAGE_CODENAMES.  Please re-visit and fix."
+                .format(cur_vers, avail_vers))
+
         fetch.apt_pkg.init()
         return fetch.apt_pkg.version_compare(avail_vers, cur_vers) == 1
 
