@@ -394,8 +394,8 @@ class TestMyOpenStackCharm(BaseOpenStackCharmTest):
         self.patch_object(chm_core.hookenv, 'apt_install')
         self.patch_object(chm_core.subprocess,
                           'check_output', return_value=b'\n')
-        self.patch_object(chm_core.os_utils, 'snap_install_requested',
-                          return_value=True)
+        self.patch_object(chm_core.os_utils, 'snap_install_requested')
+        self.snap_install_requested.return_value = True
         self.patch_object(chm_core.os_utils, 'install_os_snaps')
         self.patch_object(chm_core.os_utils,
                           'get_snaps_install_info_from_origin',
@@ -416,6 +416,11 @@ class TestMyOpenStackCharm(BaseOpenStackCharmTest):
             None,
             mode='jailmode'
         )
+        self.status_set.reset_mock()
+        self.fip.side_effect = lambda x: []
+        self.snap_install_requested.return_value = False
+        self.target.install()
+        self.assertFalse(self.status_set.called)
 
     def test_api_port(self):
         self.assertEqual(self.target.api_port('service1'), 1)
@@ -514,6 +519,35 @@ class TestMyOpenStackCharm(BaseOpenStackCharmTest):
         assert isinstance(context, MyAdapter)
         self.assertEqual(context.interfaces, ['interface1', 'interface2'])
 
+        # test source template provided with filename that represents absoulute
+        # path of target config where path separators has been replaced by
+        # underscores
+        self.render.reset_mock()
+        self.render.side_effect = [LookupError, None]
+        self.target.render_configs(
+            ['/etc/some/path1'],
+            adapters_instance=self.target.adapters_instance)
+        self.render.assert_has_calls([
+            mock.call(
+                source='path1',
+                template_loader='my-loader',
+                target='/etc/some/path1',
+                context=mock.ANY,
+                config_template=None,
+                group='root',
+                perms=0o640,
+            ),
+            mock.call(
+                source='etc_some_path1',
+                template_loader='my-loader',
+                target='/etc/some/path1',
+                context=mock.ANY,
+                config_template=None,
+                group='root',
+                perms=0o640,
+            ),
+        ])
+
     def test_render_configs_construct_adapters_instance(self):
         # give us a way to check that the context manager was called.
         from contextlib import contextmanager
@@ -782,6 +816,15 @@ class TestMyOpenStackCharm(BaseOpenStackCharmTest):
             chm_core.BaseOpenStackCharm.get_os_codename_package(
                 'testpkg', codenames),
             'newton')
+        self.upstream_version.assert_called_once_with(
+            pkg_mock.current_ver.ver_str)
+        self.upstream_version.reset_mock()
+        self.assertEqual(
+            chm_core.BaseOpenStackCharm.get_os_codename_package(
+                'testpkg', codenames, apt_cache_sufficient=True),
+            'newton')
+        self.upstream_version.assert_called_once_with(
+            pkg_mock.version)
         # Test non-fatal fail
         self.assertEqual(
             chm_core.BaseOpenStackCharm.get_os_codename_package(
@@ -791,6 +834,9 @@ class TestMyOpenStackCharm(BaseOpenStackCharmTest):
         with self.assertRaises(Exception):
             chm_core.BaseOpenStackCharm.get_os_codename_package(
                 'unknownpkg', codenames, fatal=True)
+        with self.assertRaises(ValueError):
+            chm_core.BaseOpenStackCharm.get_os_codename_package(
+                'unknownpkg', codenames, fatal=True)
 
     def test_get_os_version_package(self):
         self.patch_target('package_codenames')
